"use strict";
this.name = "oolite-random-hits-spacebar";
this.author = "LittleBear, Eric Walch & Thargoid";
this.copyright = "November 2008 - But do what you like with it in your OXPs!";
this.description	= "Checks the role of a ship attacking a spacebar to avoid friendly fire incidents between the bar's turret gunners and hunters launched by the bar."
this.licence     = "CC BY-NC-SA 3.0";
this.version = "2.3";

//2.3 cleaned ship_name related variables in ship launch

this.launchTime = clock.seconds;
this.playerWelcomed = false;
this.hopperLaunch = false; // toggled to true by hopper script.
this.quirium = ["EQ_QC_MINE", "EQ_CASCADE_MISSILE", "EQ_LAW_MISSILE", "EQ_OVERRIDE_MISSILE", "energy-bomb", "RANDOM_HITS_MINE"];
this.offenders = 0;
this.offenderCount = 0;

this.shipSpawned = function ()
{
    this.patrols = this.ship.dockedDefenders;  // set maximum number of launchable patrols
    this.hoppers = this.ship.dockedContractors;  // set maximum number of launchable hoppers
    this.maxPatrols = Math.round(Math.random()*5)+5; // 5 .. 10 out simultaneously
    this.launchHopper(); // launch the first hopper.
    this.ship.group.name = "spacebar group" ;
    this.ship.primaryRole = "random_hits_spacebar"; // all bars the same role for easier finding in scripts.
    delete this.shipSpawned;
}

this.shipBeingAttacked = function (who)
{
    if (!who) return;
    this.ship.bounty = 0;
    if (who.primaryRole === "random_hits_patrol") 
    {
        who.target = null;
        who.reactToAIMessage("TARGET_LOST"); // The AIs of the bars and hunters do not react to the nomal "ATTACKED" message, only the special "ENEMY_ATTACK". They therefore ignore attacks by their own side and the police. The lawful attacker has it's AI set to randomhitsbackoffAI which forces it to back off and stop attacking, before being returned to its normalAI.
        return;		
    }
	if (who.isPolice) 
    {
        who.target = null;
        who.reactToAIMessage("TARGET_LOST");  // Since neither police or hunter ships will deliberatley target a Clean Target - bar is kept clean at all times - The Bar takes attackes by ships with these roles as a friendly fire stray shot and does not fire back.
        return;		
    }	
		
	if (who.isPlayer) 
    {
        player.bounty |= 128;			
    }		
    
    this.ship.target = who;
    this.ship.reactToAIMessage("ENEMY_ATTACK"); // Any other attack is met with deadly force! 
};

this.doScan = function () 
{ 
	// create an array with all ships (exclude barrels and rocks) 
	function allShips(entity) {return entity.isShip && entity.scanClass !== "CLASS_ROCK" && entity.scanClass !== "CLASS_CARGO"}; 
	var anyShips = system.filteredEntities(this, allShips, this.ship, this.ship.scannerRange);
    // anyShips now holds all real ships in scanner range. From now only analyse this list further to save time.

	// do offender ships 
    var offenders = anyShips.filter(function(entity) {return entity.bounty > 5}); 
    this.offenderCount = offenders.length; // this.offenderCount is also used outside this scan function.
    if (this.offenderCount > 0){
        this.ship.target = offenders[0]; // set target to first in the list.
        this.launchPatrol(true); // priority launch
        this.ship.reactToAIMessage("OFFENDER_FOUND"); // this broacasts a groupAttackTarget.
        if (this.ship.target.bounty > 64) this.ship.alertCondition++;        

        // check for hostile offenders (Turrets shoot only at targeted hostile ships)
        // function hostiles(entity) {return entity.hasHostileTarget}; 
        var hostileShips = offenders.filter(function(entity) {return entity.hasHostileTarget}); 
        if (hostileShips.length > 0)
        {
            this.ship.target = hostileShips[0]; // set target to first in the list.
        }
    }
    
	// do missiles & mines
	// function weapons(entity) {return entity.isWeapon}; 
	var missiles = anyShips.filter(function(entity) {return entity.isWeapon}); 
    if (missiles.length > 0)
    {
        if (this.quirium.indexOf(missiles[0].primaryRole) > -1) 
        {
            this.ship.alertCondition++;
            this.ship.commsMessage(expandDescription("Quirium detected"));
            if (this.ship.minesweepers.count > 1) missiles[0].switchAI("nullAI.plist");
        }
    }

	// do player 
	//function isPlayer(entity) {return entity.isPlayer}; 
    var playerPresent = (anyShips.filter(function(entity) {return entity.isPlayer}).length > 0);
    if (playerPresent)
    {
        if (player.ship.bounty > 20)
        {
            this.ship.target = player.ship;
            this.launchPatrol(true);
            this.ship.alertCondition++;
            if (!this.playerWelcomed) this.ship.commsMessage(expandDescription("[spacebar-warning]"));
        }
        if (player.ship.bounty === 0)
        {
            this.ship.alertCondition--;
            if (!this.playerWelcomed) this.ship.commsMessage(expandDescription("[spacebar-greeting]"));
        }
        this.playerWelcomed = true;

        // 5 seconds pause means: 12 scans/minute * 8% chance = about one launch per minute.
        if (Math.random() < 0.08) this.launchPatrol(false); // random launches, no priority.
        if (this.hopperLaunch && Math.random() < 0.15) this.launchHopper();
    }
}

this.otherShipDocked = function (ship)
{
    if (!ship) return;
    
    if (ship.primaryRole === "random_hits_patrol")
    {
        this.patrols++;
    }
    if (ship.primaryRole === "random_hits_mining_hopper") 
    {
        this.hoppers++;
        this.hopperLaunch = true;
    }
    if (ship.isPlayer)
    {
        worldScripts.Random_Hits.random_hits_docked = true;
    }

}

this.stationLaunchedShip = function (ship)
{
    if (ship && ship.isPlayer)
    {
        // no need tho keep these temporary variables around after launch.
        var deletions = ["mark_system", "mark_direction", "mark_gender", "assassination_board_poster_title", "assassination_board_job_name", 
        "assassination_board_poster_surname", "assassination_board_poster_name", "assassination_board_poster_system", "assassination_board_subject",
        "assassination_board_address1", "assassination_board_address2", "assassination_board_part1", "mark_first_name", "mark_nick_name",
        "mark_second_name", "mark_race_part1", "mark_race_part2", "assassination_board_part2", "assassination_board_part3", "mark_ship_description",
        "mark_ship", "mark_ship_ad_name", "mark_ship_name", "assassination_board_part4", "assassination_board_part5", "mark_fee", "assassination_board_part6", "assassination_board_part7", 
        "billdue_name", "billdue", "security_bar_text_close", "criminal", "time_frame", "showship"];
        for (var i=0; i<deletions.length; i++) delete missionVariables["random_hits_"+deletions[i]];
        
        var that = worldScripts.Random_Hits;
        that.random_hits_docked = false;

        // When launching into a Battle near a Space Bar, Count Up the number of Enemies and tell the player.
        if (that.bigBossAttack){
            that.broadcastBigBossCount();
        }
        if (that.showScreen) delete that.showScreen; // prevents triggering missionscreens at normal stations
    }
}

/*
 Launch only patrols when the station has still some avalable. Patrols that launch are subtracted while patrols 
 that dock or leave the system are added again.
 Spread the launches with a 15 second interval.
 Also make sure there are no to many around. Only launch with a group count of less han 10 (The station and the hopper
 are included in the count)
 Keep track of the offendercount and don't do massive launches with just a few offenders around.
 */
this.launchPatrol = function (priority)
{
    var groupSize = this.ship.group.count;
    if (this.patrols > 0 && this.launchTime + groupSize*5 + 5 < clock.seconds && groupSize < this.maxPatrols+2 && groupSize < this.offenderCount + 4)
    {
        this.patrols--;
        this.launchTime = clock.seconds;
        var patrolShip = this.ship.launchShipWithRole("random_hits_patrol", priority);
        this.ship.group.addShip(patrolShip); // make it part of the space bar group
        patrolShip.group = this.ship.group;
    }
}

this.launchHopper = function ()
{
    if (this.hoppers > 0 && this.ship.alertCondition === 1)
    {
        this.hoppers--;
        var hopperShip = this.ship.launchShipWithRole("random_hits_mining_hopper");
        this.ship.group.addShip(hopperShip); // make it part of the space bar group
        hopperShip.group = this.ship.group;
        this.hopperLaunch = false;
    }
}
